@@ -1,12 +1,13 @@
-# Copyright (C) 2008, Thomas Leonard
+# Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
-import gobject
+import gobject, sys
 
+from zeroinstall.support import tasks
 from zeroinstall.injector import handler, download
 import dialog
 
-version = '0.34'
+version = '0.39'
 
 class GUIHandler(handler.Handler):
 	dl_callbacks = None		# Download -> [ callback ]
@@ -29,7 +30,7 @@ class GUIHandler(handler.Handler):
 				self.mainwindow.update_download_status()
 				return True
 			pulse()
-			self.pulse = gobject.timeout_add(50, pulse)
+			self.pulse = gobject.timeout_add(200, pulse)
 		elif len(self.monitored_downloads) == 0:
 			# Delay before resetting, in case we start a new download quickly
 			gobject.timeout_add(500, self._reset_counters)
@@ -44,10 +45,25 @@ class GUIHandler(handler.Handler):
 		self.mainwindow.update_download_status()
 
 	def confirm_trust_keys(self, interface, sigs, iface_xml):
-		import trust_box
-		return trust_box.confirm_trust(interface, sigs, iface_xml, parent = self.mainwindow.window)
+		def do_confirm():
+			import trust_box
+			return trust_box.confirm_trust(interface, sigs, iface_xml, parent = self.mainwindow.window)
+		if self.mainwindow.systray_icon:
+			self.mainwindow.systray_icon.set_tooltip('Need to confirm a new GPG key')
+			self.mainwindow.systray_icon.set_blinking(True)
+			@tasks.async
+			def wait_and_confirm():
+				# Wait for the user to click the icon, then continue
+				yield self.mainwindow.systray_icon_blocker
+				yield tasks.TimeoutBlocker(0.5, 'Delay')
+				conf = do_confirm()
+				if conf:
+					yield conf
+			return wait_and_confirm()
+		else:
+			return do_confirm()
 	
 	def report_error(self, ex, tb = None):
 		if isinstance(ex, download.DownloadAborted):
 			return		# No need to tell the user about this, since they caused it
-		dialog.alert(self.mainwindow.window, str(ex))
+		self.mainwindow.report_exception(ex)
