@@ -6,20 +6,24 @@ This module is used to invoke GnuPG to check the digital signatures on interface
 @see: L{iface_cache.PendingFeed}
 """
 
-# Copyright (C) 2006, Thomas Leonard
+# Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
 import subprocess
 import base64, re
 import os
 import tempfile
-import traceback
 from logging import info
 
-from zeroinstall import support
+from zeroinstall.support import find_in_path, basedir
 from zeroinstall.injector.trust import trust_db
 from zeroinstall.injector.model import SafeException
 
+_gnupg_options = ['gpg', '--no-secmem-warning']
+if hasattr(os, 'geteuid') and os.geteuid() == 0 and 'GNUPGHOME' not in os.environ:
+	_gnupg_options += ['--homedir', os.path.join(basedir.home, '.gnupg')]
+	info("Running as root, so setting GnuPG home to %s", _gnupg_options[-1])
+
 class Signature(object):
 	"""Abstract base class for signature check results."""
 	status = None
@@ -56,7 +60,7 @@ class ValidSig(Signature):
 	def get_details(self):
 		"""Call 'gpg --list-keys' and return the results split into lines and columns.
 		@rtype: [[str]]"""
-		child = subprocess.Popen(['gpg', '--with-colons', '--no-secmem-warning', '--list-keys', self.fingerprint], stdout = subprocess.PIPE)
+		child = subprocess.Popen(_gnupg_options + ['--with-colons', '--list-keys', self.fingerprint], stdout = subprocess.PIPE)
 		cout, unused = child.communicate()
 		if child.returncode:
 			info("GPG exited with code %d" % child.returncode)
@@ -129,7 +133,7 @@ def load_keys(fingerprints):
 	current_fpr = None
 	current_uid = None
 
-	cin, cout = os.popen2(['gpg', '--fixed-list-mode', '--with-colons', '--list-keys',
+	cin, cout = os.popen2(_gnupg_options + ['--fixed-list-mode', '--with-colons', '--list-keys',
 				'--with-fingerprint', '--with-fingerprint'] + fingerprints)
 	cin.close()
 	try:
@@ -146,6 +150,8 @@ def load_keys(fingerprints):
 					keys[current_fpr].name = current_uid
 			if line.startswith('uid:'):
 				assert current_fpr is not None
+				# Only take primary UID
+				if current_uid: continue
 				parts = line.split(':')
 				current_uid = parts[9]
 				if current_fpr in keys:
@@ -166,22 +172,10 @@ def import_key(stream):
 	"""Run C{gpg --import} with this stream as stdin."""
 	errors = tempfile.TemporaryFile()
 
-	child = os.fork()
-	if child == 0:
-		# We are the child
-		try:
-			try:
-				os.dup2(stream.fileno(), 0)
-				os.dup2(errors.fileno(), 2)
-				os.execlp('gpg', 'gpg', '--no-secmem-warning', '--quiet', '--import')
-			except:
-				traceback.print_exc()
-		finally:
-			os._exit(1)
-		assert False
+	child = subprocess.Popen(_gnupg_options + ['--quiet', '--import'],
+				stdin = stream, stderr = errors)
 
-	pid, status = os.waitpid(child, 0)
-	assert pid == child
+	status = child.wait()
 
 	errors.seek(0)
 	error_messages = errors.read().strip()
@@ -196,32 +190,20 @@ def _check_plain_stream(stream):
 
 	status_r, status_w = os.pipe()
 
-	child = os.fork()
-
-	if child == 0:
-		# We are the child
-		try:
-			try:
-				os.close(status_r)
-				os.dup2(stream.fileno(), 0)
-				os.dup2(data.fileno(), 1)
-				os.dup2(errors.fileno(), 2)
-				os.execlp('gpg', 'gpg', '--no-secmem-warning', '--decrypt',
+	# Note: Should ideally close status_r in the child, but we want to support Windows too
+	child = subprocess.Popen(_gnupg_options + ['--decrypt',
 					   # Not all versions support this:
 					   #'--max-output', str(1024 * 1024),
 					   '--batch',
-					   '--status-fd', str(status_w))
-			except:
-				traceback.print_exc()
-		finally:
-			os._exit(1)
-		assert False
-	
-	# We are the parent
+					   '--status-fd', str(status_w)],
+				stdin = stream,
+				stdout = data,
+				stderr = errors)
+
 	os.close(status_w)
 
 	try:
-		sigs = _get_sigs_from_gpg_status_stream(status_r, child, errors)
+		sigs = _get_sigs_from_gpg_status_stream(os.fdopen(status_r), child, errors)
 	finally:
 		data.seek(0)
 	return (data, sigs)
@@ -265,34 +247,20 @@ def _check_xml_stream(stream):
 		sig_file.write(sig_data)
 		sig_file.close()
 
-		status_r, status_w = os.pipe()
-
-		child = os.fork()
-
-		if child == 0:
-			# We are the child
-			try:
-				try:
-					os.close(status_r)
-					os.dup2(data.fileno(), 0)
-					os.dup2(errors.fileno(), 2)
-					os.execlp('gpg', 'gpg', '--no-secmem-warning',
+		# Note: Should ideally close status_r in the child, but we want to support Windows too
+		child = subprocess.Popen(_gnupg_options + [
 						   # Not all versions support this:
 						   #'--max-output', str(1024 * 1024),
 						   '--batch',
-						   '--status-fd', str(status_w),
-						   '--verify', sig_name, '-')
-				except:
-					traceback.print_exc()
-			finally:
-				os._exit(1)
-			assert False
-		
-		# We are the parent
-		os.close(status_w)
+						   # Windows GPG can only cope with "1" here
+						   '--status-fd', '1',
+						   '--verify', sig_name, '-'],
+						   stdin = data,
+						   stdout = subprocess.PIPE,
+						   stderr = errors)
 
 		try:
-			sigs = _get_sigs_from_gpg_status_stream(status_r, child, errors)
+			sigs = _get_sigs_from_gpg_status_stream(child.stdout, child, errors)
 		finally:
 			os.lseek(stream.fileno(), 0, 0)
 			stream.seek(0)
@@ -307,7 +275,7 @@ def check_stream(stream):
 	data is the original stream). stream must be seekable.
 	@note: Stream returned may or may not be the one passed in. Be careful!
 	@return: (data_stream, [Signatures])"""
-	if not support.find_in_path('gpg'):
+	if not find_in_path('gpg'):
 		raise SafeException("GnuPG is not installed ('gpg' not in $PATH). See http://gnupg.org")
 
 	#stream.seek(0)
@@ -336,7 +304,7 @@ def _get_sigs_from_gpg_status_stream(status_r, child, errors):
 	# Should we error out on bad signatures, even if there's a good
 	# signature too?
 
-	for line in os.fdopen(status_r):
+	for line in status_r:
 		assert line.endswith('\n')
 		assert line.startswith('[GNUPG:] ')
 		line = line[9:-1]
@@ -350,8 +318,7 @@ def _get_sigs_from_gpg_status_stream(status_r, child, errors):
 		elif code == 'ERRSIG':
 			sigs.append(ErrSig(args))
 
-	pid, status = os.waitpid(child, 0)
-	assert pid == child
+	status = child.wait()
 
 	errors.seek(0)
 
