@@ -10,16 +10,16 @@ file in memory.
 well-known variables.
 """
 
-# Copyright (C) 2006, Thomas Leonard
+# Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
 import os, re
-from logging import warn, info, debug
+from logging import info, debug
 from zeroinstall import SafeException, version
 from zeroinstall.injector.namespaces import XMLNS_IFACE
 
 # Element names for bindings in feed files
-binding_names = frozenset(['environment'])
+binding_names = frozenset(['environment', 'overlay'])
 
 network_offline = 'off-line'
 network_minimal = 'minimal'
@@ -48,14 +48,14 @@ def _split_arch(arch):
 	elif '-' not in arch:
 		raise SafeException("Malformed arch '%s'" % arch)
 	else:
-		os, machine = arch.split('-', 1)
-		if os == '*': os = None
+		osys, machine = arch.split('-', 1)
+		if osys == '*': osys = None
 		if machine == '*': machine = None
-		return os, machine
+		return osys, machine
 
-def _join_arch(os, machine):
-	if os == machine == None: return None
-	return "%s-%s" % (os or '*', machine or '*')
+def _join_arch(osys, machine):
+	if osys == machine == None: return None
+	return "%s-%s" % (osys or '*', machine or '*')
 	
 class Stability(object):
 	"""A stability rating. Each implementation has an upstream stability rating and,
@@ -94,6 +94,8 @@ def process_binding(e):
 		if not binding.name: raise InvalidInterface("Missing 'name' in binding")
 		if binding.insert is None: raise InvalidInterface("Missing 'insert' in binding")
 		return binding
+	elif e.name == 'overlay':
+		return OverlayBinding(e.getAttribute('src'), e.getAttribute('mount-point'))
 	else:
 		raise Exception("Unknown binding type '%s'" % e.name)
 
@@ -212,6 +214,32 @@ class EnvironmentBinding(Binding):
 			env_elem.setAttributeNS(None, 'default', self.default)
 		return env_elem
 
+class OverlayBinding(Binding):
+	"""Make the chosen implementation available by overlaying it onto another part of the file-system.
+	This is to support legacy programs which use hard-coded paths."""
+	__slots__ = ['src', 'mount_point']
+
+	def __init__(self, src, mount_point):
+		self.src = src
+		self.mount_point = mount_point
+
+	def __str__(self):
+		return "<overlay %s on %s>" % (self.src or '.', self.mount_point or '/')
+
+	__repr__ = __str__
+
+	def _toxml(self, doc):
+		"""Create a DOM element for this binding.
+		@param doc: document to use to create the element
+		@return: the new element
+		"""
+		env_elem = doc.createElementNS(XMLNS_IFACE, 'overlay')
+		if self.src is not None:
+			env_elem.setAttributeNS(None, 'src', self.src)
+		if self.mount_point is not None:
+			env_elem.setAttributeNS(None, 'mount-point', self.mount_point)
+		return env_elem
+
 class Feed(object):
 	"""An interface's feeds are other interfaces whose implementations can also be
 	used as implementations of this interface."""
@@ -326,7 +354,7 @@ class Implementation(object):
 
 	__slots__ = ['upstream_stability', 'user_stability', 'langs',
 		     'requires', 'main', 'metadata', 'download_sources',
-		     'id', 'feed', 'version', 'released', 'bindings']
+		     'id', 'feed', 'version', 'released', 'bindings', 'machine']
 
 	def __init__(self, feed, id):
 		assert id
@@ -341,6 +369,7 @@ class Implementation(object):
 		self.released = None
 		self.download_sources = []
 		self.langs = None
+		self.machine = None
 		self.bindings = []
 
 	def get_stability(self):
@@ -364,7 +393,7 @@ class Implementation(object):
 
 	arch = property(lambda self: _join_arch(self.os, self.machine))
 
-	os = machine = None
+	os = None
 
 class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
@@ -380,14 +409,13 @@ class DistributionImplementation(Implementation):
 class ZeroInstallImplementation(Implementation):
 	"""An implementation where all the information comes from Zero Install.
 	@since: 0.28"""
-	__slots__ = ['os', 'machine', 'size']
+	__slots__ = ['os', 'size']
 
 	def __init__(self, feed, id):
 		"""id can be a local path (string starting with /) or a manifest hash (eg "sha1=XXX")"""
 		Implementation.__init__(self, feed, id)
 		self.size = None
 		self.os = None
-		self.machine = None
 
 	# Deprecated
 	dependencies = property(lambda self: dict([(x.interface, x) for x in self.requires
@@ -631,9 +659,11 @@ class ZeroInstallFeed(object):
 
 			impl.metadata = item_attrs
 			try:
-				version = item_attrs['version']
 				version_mod = item_attrs.get('version-modifier', None)
-				if version_mod: version += version_mod
+				if version_mod:
+					item_attrs['version'] += version_mod
+					del item_attrs['version-modifier']
+				version = item_attrs['version']
 			except KeyError:
 				raise InvalidInterface("Missing version attribute")
 			impl.version = parse_version(version)
@@ -721,11 +751,10 @@ class ZeroInstallFeed(object):
 
 			distro.get_package_info(package, factory)
 		
-		process_group(feed_element,
-			{'stability': 'testing',
-			 'main' : main,
-			},
-			[], [])
+		root_attrs = {'stability': 'testing'}
+		if main:
+			root_attrs['main'] = main
+		process_group(feed_element, root_attrs, [], [])
 
 	def get_name(self):
 		return self.name or '(' + os.path.basename(self.url) + ')'
@@ -800,12 +829,17 @@ def _pretty_escape(uri):
 
 def canonical_iface_uri(uri):
 	"""If uri is a relative path, convert to an absolute one.
+	A "file:///foo" URI is converted to "/foo".
 	Otherwise, return it unmodified.
 	@rtype: str
 	@raise SafeException: if uri isn't valid
 	"""
-	if uri.startswith('http:'):
+	if uri.startswith('http://'):
+		if uri.find("/", 7) == -1:
+			raise SafeException("Missing / after hostname in URI '%s'" % uri)
 		return uri
+	elif uri.startswith('file:///'):
+		return uri[7:]
 	else:
 		iface_uri = os.path.realpath(uri)
 		if os.path.isfile(iface_uri):
