@@ -4,20 +4,20 @@ L{fetch.Fetcher} to download additional components, and the user's configuration
 settings.
 """
 
-# Copyright (C) 2008, Thomas Leonard
+# Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
 import time
-import sys, os
+import os
 from logging import info, debug, warn
-import arch
-
-from model import *
-from namespaces import *
 import ConfigParser
+
+from zeroinstall import SafeException
+from zeroinstall.injector import arch
+from zeroinstall.injector.model import Interface, Implementation, network_levels, network_offline, DistributionImplementation, network_full
+from zeroinstall.injector.namespaces import config_site, config_prog, injector_gui_uri
 from zeroinstall.support import tasks, basedir
-from zeroinstall.injector.iface_cache import iface_cache, PendingFeed
-from zeroinstall.injector.trust import trust_db
+from zeroinstall.injector.iface_cache import iface_cache
 
 # If we started a check within this period, don't start another one:
 FAILED_CHECK_DELAY = 60 * 60	# 1 Hour
@@ -31,6 +31,8 @@ class Policy(object):
 	 4. Use L{get_uncached_implementations} to find where to get these versions and download them
 	    using L{download_uncached_implementations}.
 
+	@ivar target_arch: target architecture for binaries
+	@type target_arch: L{arch.Architecture}
 	@ivar root: URI of the root interface
 	@ivar solver: solver used to choose a set of implementations
 	@type solver: L{solve.Solver}
@@ -49,7 +51,7 @@ class Policy(object):
 	"""
 	__slots__ = ['root', 'watchers',
 		     'freshness', 'handler', '_warned_offline',
-		     'src', 'stale_feeds', 'solver', '_fetcher']
+		     'target_arch', 'src', 'stale_feeds', 'solver', '_fetcher']
 	
 	help_with_testing = property(lambda self: self.solver.help_with_testing,
 				     lambda self, value: setattr(self.solver, 'help_with_testing', value))
@@ -103,6 +105,8 @@ class Policy(object):
 
 		self.set_root(root)
 
+		self.target_arch = arch.get_host_architecture()
+
 	@property
 	def fetcher(self):
 		if not self._fetcher:
@@ -134,7 +138,7 @@ class Policy(object):
 		"""@deprecated: see L{solve_with_downloads} """
 		self.stale_feeds = set()
 
-		host_arch = arch.get_host_architecture()
+		host_arch = self.target_arch
 		if self.src:
 			host_arch = arch.SourceArchitecture(host_arch)
 		self.solver.solve(self.root, host_arch)
@@ -267,7 +271,7 @@ class Policy(object):
 	
 	def get_uncached_implementations(self):
 		"""List all chosen implementations which aren't yet available locally.
-		@rtype: [(str, model.Implementation)]"""
+		@rtype: [(L{model.Interface}, L{model.Implementation})]"""
 		uncached = []
 		for iface in self.solver.selections:
 			impl = self.solver.selections[iface]
@@ -310,7 +314,7 @@ class Policy(object):
 		downloads_finished = set()		# Successful or otherwise
 		downloads_in_progress = {}		# URL -> Download
 
-		host_arch = arch.get_host_architecture()
+		host_arch = self.target_arch
 		if self.src:
 			host_arch = arch.SourceArchitecture(host_arch)
 
@@ -321,18 +325,20 @@ class Policy(object):
 			if self.solver.ready and not force:
 				break
 			else:
+				if self.network_use == network_offline and not force:
+					info("Can't choose versions and in off-line mode, so aborting")
+					break
 				# Once we've starting downloading some things,
 				# we might as well get them all.
 				force = True
 
-			if not self.network_use == network_offline:
-				for f in self.solver.feeds_used:
-					if f in downloads_finished or f in downloads_in_progress:
-						continue
-					if f.startswith('/'):
-						continue
-					feed = iface_cache.get_interface(f)
-					downloads_in_progress[f] = self.fetcher.download_and_import_feed(f, iface_cache)
+			for f in self.solver.feeds_used:
+				if f in downloads_finished or f in downloads_in_progress:
+					continue
+				if f.startswith('/'):
+					continue
+				feed = iface_cache.get_interface(f)
+				downloads_in_progress[f] = self.fetcher.download_and_import_feed(f, iface_cache)
 
 			if not downloads_in_progress:
 				break
@@ -350,7 +356,7 @@ class Policy(object):
 		"""Decide whether we need to download anything (but don't do it!)
 		@return: true if we MUST download something (feeds or implementations)
 		@rtype: bool"""
-		host_arch = arch.get_host_architecture()
+		host_arch = self.target_arch
 		if self.src:
 			host_arch = arch.SourceArchitecture(host_arch)
 		self.solver.solve(self.root, host_arch)
