@@ -2,7 +2,7 @@
 Code for managing the implementation cache.
 """
 
-# Copyright (C) 2006, Thomas Leonard
+# Copyright (C) 2009, Thomas Leonard
 # See the README file for details, or visit http://0install.net.
 
 import os
@@ -237,7 +237,7 @@ class Stores(object):
 		"""Search for digest in all stores."""
 		assert digest
 		if '/' in digest or '=' not in digest:
-			raise BadDigest('Syntax error in digest (use ALG=VALUE)')
+			raise BadDigest('Syntax error in digest (use ALG=VALUE, not %s)' % digest)
 		for store in self.stores:
 			path = store.lookup(digest)
 			if path:
